package org.xbib.j2html.tags;

import org.xbib.j2html.Config;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Scanner;

import static org.xbib.j2html.TagCreator.rawHtml;
import static org.xbib.j2html.TagCreator.script;
import static org.xbib.j2html.TagCreator.style;

public class InlineStaticResource {

    public static ContainerTag<? extends Tag<?>> get(String path, TargetFormat format) {
        String fileString = getFileAsString(path);
        return switch (format) {
            case CSS_MIN -> style().with(rawHtml(Config.cssMinifier.minify(fileString)));
            case JS_MIN -> script().with(rawHtml(Config.jsMinifier.minify((fileString))));
            case CSS -> style().with(rawHtml(fileString));
            case JS -> script().with(rawHtml(fileString));
        };
    }

    public static String getFileAsString(String path) {
        try {
            return streamToString(InlineStaticResource.class.getResourceAsStream(path));
        } catch (Exception expected) {
            try {
                return streamToString(new FileInputStream(path));
            } catch (Exception exception) {
                throw new RuntimeException("Couldn't find file with path='" + path + "'");
            }
        }
    }

    private static String streamToString(InputStream inputStream) {
        Scanner s = new Scanner(inputStream).useDelimiter("\\A");
        return s.hasNext() ? s.next() : "";
    }

    public enum TargetFormat {CSS_MIN, CSS, JS_MIN, JS}

}

